@@ -83,7 +83,7 @@ gconf_key_directory  (const gchar* key)
 
   if (end == NULL)
     {
-      gconf_log(GCL_ERR, _("No '/' in key `%s'"), key);
+      gconf_log(GCL_ERR, _("No '/' in key \"%s\""), key);
       return NULL;
     }
 
@@ -885,7 +885,7 @@ gconf_load_source_path(const gchar* filename, GError** err)
           if (*varsubst != '\0') /* Drop lines with just two quote marks or something */
             {
               gconf_log(GCL_DEBUG, _("Adding source `%s'\n"), varsubst);
-              l = g_slist_prepend(l, g_strdup(varsubst));
+              l = g_slist_append (l, g_strdup(varsubst));
             }
           g_free(varsubst);
         }
@@ -2394,7 +2394,7 @@ open_empty_locked_file (const gchar *directory,
       g_set_error (err,
                    GCONF_ERROR,
                    GCONF_ERROR_LOCK_FAILED,
-                   _("Failed to lock '%s': another process has the lock (%s)"),
+                   _("Failed to lock '%s': probably another process has the lock, or your operating system has NFS file locking misconfigured (%s)"),
                    filename, strerror (errno));
       close (fd);
       return -1;
@@ -2710,7 +2710,8 @@ const char *
 get_hostname (void)
 {
 	static char *hostname = NULL;
-	char hn_tmp[65], ha_tmp[4];
+	char hn_tmp[65];
+	gulong ha_tmp;
 	struct hostent *hent;
 
 	if (!hostname) {
@@ -2718,8 +2719,8 @@ get_hostname (void)
 
 		hent = gethostbyname (hn_tmp);
 		if (hent) {
-			memcpy (ha_tmp, hent->h_addr, 4);
-			hent = gethostbyaddr (ha_tmp, 4, AF_INET);
+			memcpy (&ha_tmp, hent->h_addr, 4);
+			hent = gethostbyaddr ((char *)&ha_tmp, 4, AF_INET);
 			if (hent)
 				hostname = g_strdup (hent->h_name);
 			else
@@ -2727,7 +2728,7 @@ get_hostname (void)
 					g_strdup (inet_ntoa
 						  (*
 						   ((struct in_addr *)
-						    ha_tmp)));
+						    &ha_tmp)));
 		} else
 			hostname = g_strdup (hn_tmp);
 	}
@@ -2735,49 +2736,53 @@ get_hostname (void)
 	return hostname;
 }
 
+static CORBA_ORB gconf_orb = CORBA_OBJECT_NIL;      
+
 CORBA_ORB
 gconf_orb_get (void)
 {
-  if (TRUE || gconf_in_daemon_mode ())
+  if (gconf_orb == CORBA_OBJECT_NIL)
     {
-      static CORBA_ORB gconf_orb = CORBA_OBJECT_NIL;      
+      CORBA_Environment ev;
+      int argc = 1;
+      char *argv[] = { "gconf", NULL };
+      
+      CORBA_exception_init (&ev);
+      
+      gconf_orb = CORBA_ORB_init (&argc, argv, "orbit-local-orb", &ev);
+      g_assert (ev._major == CORBA_NO_EXCEPTION);
 
-      if (gconf_orb == CORBA_OBJECT_NIL)
+      CORBA_exception_free (&ev);
+    }
+
+  return gconf_orb;
+}
+
+int
+gconf_orb_release (void)
+{
+  int ret = 0;
+
+  if (gconf_orb != CORBA_OBJECT_NIL)
+    {
+      CORBA_ORB orb = gconf_orb;
+      CORBA_Environment ev;
+
+      gconf_orb = CORBA_OBJECT_NIL;
+
+      CORBA_exception_init (&ev);
+
+      CORBA_ORB_destroy (orb, &ev);
+      CORBA_Object_release ((CORBA_Object)orb, &ev);
+
+      if (ev._major != CORBA_NO_EXCEPTION)
         {
-          CORBA_Environment ev;
-          int argc = 1;
-          char *argv[] = { "gconf", NULL };
-          CORBA_Context context;
-          const char *hostname;
-      
-          CORBA_exception_init (&ev);
-      
-          gconf_orb = CORBA_ORB_init (&argc, argv, "orbit-local-orb", &ev);
-          g_assert (ev._major == CORBA_NO_EXCEPTION);
-
-          /* Set values in default context */
-          CORBA_ORB_get_default_context (gconf_orb, &context, &ev);
-          g_assert (ev._major == CORBA_NO_EXCEPTION);
-          
-          hostname = get_hostname ();
-          CORBA_Context_set_one_value (context, "hostname",
-                                       (char *) hostname, &ev);
-          CORBA_Context_set_one_value (context, "domain", "user", &ev);
-          CORBA_Context_set_one_value (context, "username",
-                                       g_get_user_name (), &ev);
-          
-          CORBA_exception_free (&ev);
+          ret = 1;
         }
-      
-      return gconf_orb;
-    }
-  else
-    {
-      /* FIXME do we have to do this if an app is using bonobo activation? */
-#if 0
-      return bonobo_activation_orb_get ();
-#endif
+      CORBA_exception_free (&ev);
     }
+
+  return ret;
 }
 
 char*
@@ -2919,7 +2924,7 @@ gconf_activate_server (gboolean  start_if_not_found,
     g_set_error (error,
                  GCONF_ERROR,
                  GCONF_ERROR_NO_SERVER,
-                 _("Failed to contact configuration server (a likely cause of this is that you have an existing configuration server (gconfd) running, but it isn't reachable from here - if you're logged in from two machines at once, you may need to enable TCP networking for ORBit)\n"));
+                 _("Failed to contact configuration server; some possible causes are:\na) you have an existing configuration server (gconfd) running, but it isn't reachable from here - if you're logged in from two machines at once, you may need to enable TCP networking for ORBit by putting the line \"ORBIIOPIPv4=1\" in /etc/orbitrc\nb) you have stale locks in your NFS-mounted home directory due to a system crash, try removing ~/.gconf/*.lock and ~/.gconfd/lock if you are sure no gconfd processes are running on any machine using your home dir"));
 
   close (p[0]);
   close (p[1]);
