@@ -25,7 +25,7 @@
 #include <popt.h>
 #include <libxml/tree.h>
 #include <libxml/parser.h>
-#include <libxml/xmlmemory.h>
+#include <libxml/globals.h>
 #include <stdlib.h>
 #include <errno.h>
 
@@ -57,7 +57,9 @@ static int short_docs_mode = FALSE;
 static int long_docs_mode = FALSE;
 static int schema_name_mode = FALSE;
 static int associate_schema_mode = FALSE;
+static int dissociate_schema_mode = FALSE;
 static int default_source_mode = FALSE;
+static int recursive_unset_mode = FALSE;
 
 struct poptOption options[] = {
   { 
@@ -106,6 +108,15 @@ struct poptOption options[] = {
     N_("Unset the keys on the command line"),
     NULL
   },
+  {
+    "recursive-unset",
+    '\0',
+    POPT_ARG_NONE,
+    &recursive_unset_mode,
+    0, 
+    N_("Recursively unset all keys at or below the key/directory names on the command line"),
+    NULL
+  },
   { 
     "all-entries",
     'a',
@@ -322,6 +333,15 @@ struct poptOption options[] = {
     N_("Specify the schema name followed by the key to apply the schema name to"),
     NULL
   },
+  {
+    "unapply-schema",
+    '\0',
+    POPT_ARG_NONE,
+    &dissociate_schema_mode,
+    0,
+    N_("Remove any schema name applied to the given keys"),
+    NULL
+  },
   {
     "get-default-source",
     '\0',
@@ -355,12 +375,14 @@ static int do_set(GConfEngine* conf, const gchar** args);
 static int do_set_schema(GConfEngine* conf, const gchar** args);
 static int do_all_entries(GConfEngine* conf, const gchar** args);
 static int do_unset(GConfEngine* conf, const gchar** args);
+static int do_recursive_unset (GConfEngine* conf, const gchar** args);
 static int do_all_subdirs(GConfEngine* conf, const gchar** args);
 static int do_load_schema_file(GConfEngine* conf, const gchar* file);
 static int do_short_docs (GConfEngine *conf, const gchar **args);
 static int do_long_docs (GConfEngine *conf, const gchar **args);
 static int do_get_schema_name (GConfEngine *conf, const gchar **args);
 static int do_associate_schema (GConfEngine *conf, const gchar **args);
+static int do_dissociate_schema (GConfEngine *conf, const gchar **args);
 static int do_get_default_source (GConfEngine *conf, const gchar **args);
 
 int 
@@ -517,12 +539,19 @@ main (int argc, char** argv)
       return 1;
     }
 
+  /* FIXME not checking that --recursive-unset is used alone */
   
   if (use_local_source && config_source == NULL)
     {
       fprintf(stderr, _("You must specify a config source with --config-source when using --direct\n"));
       return 1;
     }
+
+  if (config_source && !use_local_source)
+    {
+      fprintf (stderr, _("You should use --direct when using a non-default configuration source\n"));
+      return 1;
+    }
   
   if (!gconf_init(argc, argv, &err))
     {
@@ -541,13 +570,30 @@ main (int argc, char** argv)
       else 
         return 2;
     }
+
+  /* Before creating engine */
+  if (default_source_mode)
+    {
+      const gchar** args = poptGetArgs(ctx);
+      if (do_get_default_source (conf, args)  == 1)
+        return 1;
+      else
+        return 0;
+    }
+
+  if (makefile_install_mode &&
+      g_getenv ("GCONF_DISABLE_MAKEFILE_SCHEMA_INSTALL"))
+    {
+      g_print (_("GCONF_DISABLE_MAKEFILE_SCHEMA_INSTALL is set, not installing schemas\n"));
+      makefile_install_mode = FALSE;
+    }
   
   if (makefile_install_mode)
     {
       g_assert (config_source == NULL);
 
       /* Try the environment variable */
-      config_source = g_getenv("GCONF_CONFIG_SOURCE");
+      config_source = g_getenv ("GCONF_CONFIG_SOURCE");
 
       if (config_source == NULL)
         {
@@ -562,6 +608,9 @@ main (int argc, char** argv)
         }
 
       use_local_source = TRUE;
+
+      /* shut down daemon, this is a race condition, but will usually work. */
+      gconf_shutdown_daemon (NULL);
     }
   
   if (config_source == NULL)
@@ -624,9 +673,9 @@ main (int argc, char** argv)
   if (makefile_install_mode)
     {
       const gchar** args = poptGetArgs(ctx);
-      gint retval = do_makefile_install(conf, args);
-
-      gconf_engine_unref(conf);
+      gint retval = do_makefile_install (conf, args);
+      
+      gconf_engine_unref (conf);
 
       return retval;
     }
@@ -722,12 +771,12 @@ main (int argc, char** argv)
         }
     }
 
-  if (default_source_mode)
+  if (dissociate_schema_mode)
     {
-      const gchar** args = poptGetArgs(ctx);
-      if (do_get_default_source(conf, args)  == 1)
+      const gchar** args = poptGetArgs (ctx);
+      if (do_dissociate_schema (conf, args)  == 1)
         {
-          gconf_engine_unref(conf);
+          gconf_engine_unref (conf);
           return 1;
         }
     }
@@ -754,6 +803,17 @@ main (int argc, char** argv)
         }
     }
 
+  if (recursive_unset_mode)
+    {
+      const gchar** args = poptGetArgs(ctx);
+
+      if (do_recursive_unset(conf, args) == 1)
+        {
+          gconf_engine_unref(conf);
+          return 1;
+        }
+    }
+  
   if (all_subdirs_mode)
     {
       const gchar** args = poptGetArgs(ctx);
@@ -1361,6 +1421,39 @@ do_associate_schema (GConfEngine *conf, const gchar **args)
   return 0;
 }
 
+static int
+do_dissociate_schema (GConfEngine *conf, const gchar **args)
+{
+  GError *err;
+  gboolean failed;
+  
+  if (args[0] == NULL)
+    {
+      fprintf (stderr, _("Must specify a schema name followed by the key name to apply it to\n"));
+      return 1;
+    }
+
+  failed = FALSE;
+  while (*args)
+    {
+      err = NULL;
+      if (!gconf_engine_associate_schema (conf, *args, NULL, &err))
+        {
+          fprintf (stderr, _("Error removing schema name from '%s': %s\n"),
+                   *args, err->message);
+          g_error_free (err);
+          failed = TRUE;
+        }
+      
+      ++args;
+    }
+
+  if (failed)
+    return 1;
+  else
+    return 0;
+}
+
 static int
 do_set_schema(GConfEngine* conf, const gchar** args)
 {
@@ -1537,6 +1630,35 @@ do_unset(GConfEngine* conf, const gchar** args)
   return 0;
 }
 
+static int
+do_recursive_unset (GConfEngine* conf, const gchar** args)
+{
+  if (args == NULL)
+    {
+      fprintf (stderr, _("Must specify one or more keys to recursively unset.\n"));
+      return 1;
+    }  
+      
+  while (*args)
+    {
+      GError *err;
+
+      err = NULL;
+      gconf_engine_recursive_unset (conf, *args,
+                                    GCONF_UNSET_INCLUDING_SCHEMA_NAMES,
+                                    &err);
+      if (err != NULL)
+        {
+          fprintf (stderr, _("Failure during recursive unset of \"%s\": %s\n"),
+                   *args, err->message);
+          g_error_free (err);
+        }
+      
+      ++args;
+    }
+
+  return 0;
+}
 
 static int
 do_all_subdirs(GConfEngine* conf, const gchar** args)
@@ -2152,12 +2274,15 @@ process_schema_list(GConfEngine* conf, xmlNodePtr node)
 
   while (iter != NULL)
     {
-      if (strcmp(iter->name, "schema") == 0)
-        process_schema(conf, iter);
-      else
-        fprintf(stderr, _("WARNING: node <%s> not understood below <schemalist>\n"),
-                iter->name);
-
+      if (iter->type == XML_ELEMENT_NODE)
+        {
+          if (strcmp(iter->name, "schema") == 0)
+            process_schema(conf, iter);
+          else
+            fprintf(stderr, _("WARNING: node <%s> not understood below <schemalist>\n"),
+                    iter->name);
+        }
+      
       iter = iter->next;
     }
 
