@@ -17,8 +17,37 @@
  * Boston, MA 02111-1307, USA.
  */
 
+
+
+
+
+
+
+
+
+
+
+/*
+ *
+ *
+ * DO NOT USE THESE CRAPPY TESTS AS EXAMPLE CODE. USE THE DOCS AND examples
+ *
+ *
+ *
+ */
+
+
+
+
+
+
+
+
+
 #include <gconf/gconf.h>
 #include <stdio.h>
+#include <string.h>
+#include <stdlib.h>
 #include <unistd.h>
 #include <math.h>
 #include <gconf/gconf-internals.h>
@@ -150,6 +179,16 @@ check_unset(GConfEngine* conf)
 
   while (*keyp)
     {
+      gconf_engine_associate_schema (conf, *keyp, "/test-bogus-schema", &err);
+
+      if (err != NULL)
+        {
+          fprintf(stderr, "unset of `%s' failed: %s\n", *keyp, err->message);
+          g_error_free(err);
+          err = NULL;
+          exit (1);
+        }
+      
       gconf_engine_unset(conf, *keyp, &err);
 
       if (err != NULL)
@@ -157,6 +196,7 @@ check_unset(GConfEngine* conf)
           fprintf(stderr, "unset of `%s' failed: %s\n", *keyp, err->message);
           g_error_free(err);
           err = NULL;
+          exit (1);
         }
       else
         {
@@ -931,10 +971,12 @@ main (int argc, char** argv)
   printf("\nChecking bool storage:");
   
   check_bool_storage(conf);
-  
+
+  gconf_engine_set_bool(conf, "/foo", TRUE, &err);
+
   gconf_engine_unref(conf);
 
   printf("\n\n");
   
-  return 0;
+  return gconf_debug_shutdown ();
 }
